package org.klomp.eclipse.team.sftp.client;

import java.io.IOException;

import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.team.internal.ccvs.core.connection.CVSAuthenticationException;
import org.eclipse.team.internal.ccvs.ssh.Client;
import org.klomp.eclipse.team.sftp.SftpException;
import org.klomp.eclipse.team.sftp.SftpPlugin;
import org.klomp.sftp.protocol.SftpConnection;

public class Ssh1SftpConnection extends SftpConnection
{

    private String serverCmd;
    private String host;
    private int port;

    private String user;

    private int timeout = -1;

    /**
     * password for user specified in repository location string
     */
    private String password;

    /**
     * ssh client
     */
    private Client client;

    protected Ssh1SftpConnection(String host, int port, String serverCmd, String user, String password)
    {

        this.host = host;
        this.port = port;
        if (serverCmd == null)
            serverCmd = SftpPlugin.DEFAULT_SERVER_CMD;
        this.serverCmd = serverCmd;

        this.user = user;
        this.password = password;
    }

    public void close() throws IOException
    {
        client.disconnect();
        if (listener != null)
        {
            listener.connectionClosed();
        }
    }

    /**
     * Opens the connection and invokes cvs in server mode.
     *
     * @see Connection.open()
     */
    public void open(int timeout, IProgressMonitor monitor) throws SftpException
    {
        monitor.subTask("Connecting...");
        monitor.worked(1);
        // create the connection using host, username, and password
        client = new Client(host, port, user, password, serverCmd, timeout);
        try
        {
            client.connect(monitor);
            input = client.getInputStream();
            output = client.getOutputStream();
        }
        catch (CVSAuthenticationException e)
        {
            throw new SftpException(e.getMessage(), e);
        }
        catch (IOException e)
        {
            throw new SftpCommunicationException(e.getMessage(), e);
        }

        if (listener != null)
        {
            listener.connectionEstablished(host, port, user);
        }
    }

}
