package org.klomp.eclipse.team.sftp.client;

import java.io.IOException;

import org.eclipse.core.runtime.IProgressMonitor;

import com.sshtools.j2ssh.SshClient;
import com.sshtools.j2ssh.authentication.PasswordAuthenticationClient;
import com.sshtools.j2ssh.configuration.SshConnectionProperties;
import com.sshtools.j2ssh.connection.Channel;
import com.sshtools.j2ssh.session.SessionChannelClient;
import com.sshtools.j2ssh.transport.HostKeyVerification;
import com.sshtools.j2ssh.transport.TransportProtocolException;
import com.sshtools.j2ssh.transport.publickey.SshPublicKey;
import org.klomp.eclipse.team.sftp.SftpException;
import org.klomp.sftp.protocol.SftpConnection;

public class Ssh2SftpConnection extends SftpConnection
{
    private String host;
    private int port;

    private String user;

    private int timeout = -1;

    /**
     * password for user specified in repository location string
     */
    private String password;

    private SshClient client;
    private Channel sftpChannel;

    protected Ssh2SftpConnection(String host, int port, String user, String password)
    {

        this.host = host;
        this.port = port;
        this.user = user;
        this.password = password;
    }

    public void close() throws IOException
    {
        if (sftpChannel != null)
        {
            sftpChannel.close();
        }
        if (client != null)
        {
            client.disconnect();
        }
        if (listener != null)
        {
            listener.connectionClosed();
        }
    }

    public void open(int timeout, IProgressMonitor monitor) throws SftpException
    {
        monitor.subTask("Connecting...");
        monitor.worked(1);

        try
        {
            client = new SshClient();

            SshConnectionProperties properties = new SshConnectionProperties();
            properties.setHost(host);
            properties.setPort(port);
            properties.setUsername(user);
            PasswordAuthenticationClient passwdAuth = new PasswordAuthenticationClient();
            passwdAuth.setUsername(user);
            passwdAuth.setPassword(password);

            HostKeyVerification verification = new HostKeyVerification()
            {
                public boolean verifyHost(String host, SshPublicKey pk) throws TransportProtocolException
                {
                    return true; // TOD implement safe protocol
                }
            };

            client.connect(properties, verification);
            client.authenticate(passwdAuth);

            SessionChannelClient channel = client.openSessionChannel();
            channel.startSubsystem("sftp");
            this.sftpChannel = channel;

            input = channel.getInputStream();
            output = channel.getOutputStream();
        }
        catch (IOException e)
        {
            throw new SftpCommunicationException(e.getMessage(), e);
        }

        if (listener != null)
        {
            listener.connectionEstablished(host, port, user);
        }
    }
}
