package org.klomp.eclipse.team.sftp.target;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.team.core.TeamException;
import org.klomp.eclipse.team.sftp.client.ISftpRunnable;
import org.klomp.eclipse.team.sftp.client.SftpServerException;
import org.klomp.sftp.packet.FxpStatus;

public class CreateDirectoriesRunnable implements ISftpRunnable
{
    private SftpTargetProvider provider;
    private IPath relativePath;

    public CreateDirectoriesRunnable(SftpTargetProvider provider, IPath relativePath)
    {
        this.provider = provider;
        this.relativePath = relativePath;
    }

    public void run(String basePath, IProgressMonitor monitor) throws TeamException
    {
        try
        {
            provider.getClient().createDirectory(basePath + relativePath.toString(), monitor);
        }
        catch (SftpServerException e)
        {
            if (e.getStatus().getCode() == FxpStatus.FX_FAILURE)
            {
                // A parent must not exist, try to create it
                provider.createDirectory(relativePath.removeLastSegments(1), monitor);
                provider.getClient().createDirectory(basePath + relativePath.toString(), monitor);
            }
            else
            {
                throw e;
            }
        }
    }

}
