package org.klomp.eclipse.team.sftp.target;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.sync.ILocalSyncElement;
import org.eclipse.team.core.sync.IRemoteResource;
import org.eclipse.team.core.sync.IRemoteSyncElement;
import org.eclipse.team.internal.core.target.IRemoteTargetResource;
import org.eclipse.team.internal.core.target.RemoteTargetSyncElement;
import org.eclipse.team.internal.core.target.TargetProvider;
import org.klomp.eclipse.team.sftp.SftpPlugin;

public class SftpRemoteSyncElement extends RemoteTargetSyncElement
{
    public SftpRemoteSyncElement(TargetProvider provider, IResource local, IRemoteTargetResource remote)
    {
        super(provider, local, remote);
    }

    public IRemoteSyncElement create(
        boolean isThreeWay,
        IResource local,
        IRemoteResource base,
        IRemoteResource remote,
        Object data)
    {
        return new SftpRemoteSyncElement(getProvider(), local, (IRemoteTargetResource) remote);
    }

    public ILocalSyncElement create(IResource local, IRemoteResource base, Object data)
    {
        return new SftpRemoteSyncElement(getProvider(), local, (IRemoteTargetResource) base);
    }

    protected boolean isOutOfDate(IProgressMonitor monitor)
    {

        SftpTargetProvider provider = (SftpTargetProvider) getProvider();
        try
        {
            SftpResourceState state =
                (SftpResourceState) provider.getState(getLocal(), (IRemoteTargetResource) getRemote());
            return state.wasOutOfDate(monitor);
        }
        catch (TeamException e)
        {
            SftpPlugin.log(e);
            return true; // who knows?
        }
    }

}
