package org.klomp.sftp.packet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.klomp.ssh.DataIO;

public abstract class SftpPacket
{
    public final static byte FXP_INIT = 1;
    public final static byte FXP_VERSION = 2;
    public final static byte FXP_OPEN = 3;
    public final static byte FXP_CLOSE = 4;
    public final static byte FXP_READ = 5;
    public final static byte FXP_WRITE = 6;
    public final static byte FXP_LSTAT = 7;
    public final static byte FXP_FSTAT = 8;
    public final static byte FXP_SETSTAT = 9;
    public final static byte FXP_FSETSTAT = 10;
    public final static byte FXP_OPENDIR = 11;
    public final static byte FXP_READDIR = 12;
    public final static byte FXP_REMOVE = 13;
    public final static byte FXP_MKDIR = 14;
    public final static byte FXP_RMDIR = 15;
    public final static byte FXP_REALPATH = 16;
    public final static byte FXP_STAT = 17;
    public final static byte FXP_RENAME = 18;
    public final static byte FXP_READLINK = 19;
    public final static byte FXP_SYMLINK = 20;
    public final static byte FXP_STATUS = 101;
    public final static byte FXP_HANDLE = 102;
    public final static byte FXP_DATA = 103;
    public final static byte FXP_NAME = 104;
    public final static byte FXP_ATTRS = 105;
    public final static byte FXP_EXTENDED = (byte) 200;
    public final static byte FXP_EXTENDED_REPLY = (byte) 201;

    private byte type;
    protected int version;
    protected int id;

    public static String getPacketTypeName(int type)
    {
        switch (type)
        {
            case SftpPacket.FXP_INIT :
                return "INIT";
            case SftpPacket.FXP_VERSION :
                return "VERSION";
            case SftpPacket.FXP_OPEN :
                return "OPEN";
            case SftpPacket.FXP_CLOSE :
                return "CLOSE";
            case SftpPacket.FXP_READ :
                return "READ";
            case SftpPacket.FXP_WRITE :
                return "WRITE";
            case SftpPacket.FXP_LSTAT :
                return "LSTAT";
            case SftpPacket.FXP_FSTAT :
                return "FSTAT";
            case SftpPacket.FXP_SETSTAT :
                return "SETSTAT";
            case SftpPacket.FXP_FSETSTAT :
                return "FSETSTAT";
            case SftpPacket.FXP_OPENDIR :
                return "OPENDIR";
            case SftpPacket.FXP_READDIR :
                return "READDIR";
            case SftpPacket.FXP_REMOVE :
                return "REMOVE";
            case SftpPacket.FXP_MKDIR :
                return "MKDIR";
            case SftpPacket.FXP_RMDIR :
                return "RMDIR";
            case SftpPacket.FXP_REALPATH :
                return "REALPATH";
            case SftpPacket.FXP_STAT :
                return "STAT";
            case SftpPacket.FXP_RENAME :
                return "RENAME";
            case SftpPacket.FXP_READLINK :
                return "READLINK";
            case SftpPacket.FXP_SYMLINK :
                return "SYMLINK";
            case SftpPacket.FXP_STATUS :
                return "STATUS";
            case SftpPacket.FXP_HANDLE :
                return "HANDLE";
            case SftpPacket.FXP_DATA :
                return "DATA";
            case SftpPacket.FXP_NAME :
                return "NAME";
            case SftpPacket.FXP_ATTRS :
                return "ATTRS";
            case SftpPacket.FXP_EXTENDED :
                return "EXTENDED";
            case SftpPacket.FXP_EXTENDED_REPLY :
                return "EXTENDED_REPLY";
            default :
                return "UNKNOWN(" + type + ")";
        }
    }

    protected SftpPacket(byte type, int version)
    {
        this.type = type;
        this.version = version;
    }

    protected SftpPacket(byte type)
    {
        this.type = type;
    }

    protected SftpPacket(byte type, int version, int id)
    {
        this.type = type;
        this.version = version;
        this.id = id;
    }

    protected abstract void readPayload(InputStream payload) throws IOException;
    protected abstract void writePayload(ByteArrayOutputStream payload);

    public String toString()
    
    {
        return getId() + " " + getPacketTypeName(type);
    }

    public byte getType()
    {
        return type;
    }

    /**
     * Creates a packet with room for the given payload size. The length and
     * type fields of the packet will be filled in.
     */
    protected ByteArrayOutputStream createPacketStream()
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataIO.writeInt(out, 0);
        out.write(type);
        return out;
    }

    public static byte[] createPacket(ByteArrayOutputStream out)
    {
        byte[] packet = out.toByteArray();
        //setInt(packet, 0, packet.length - 5);
        DataIO.setInt(packet, 0, packet.length - 4);
        return packet;
    }

    public void writePacket(OutputStream out) throws IOException
    {
        ByteArrayOutputStream packetStream = createPacketStream();
        writePayload(packetStream);
        byte[] packet = createPacket(packetStream);
        out.write(packet);
        out.flush();
    }

    public final void setId(int id)
    {
        this.id = id;
    }

    public final int getId()
    {
        return id;
    }

}
