package org.klomp.sftp.protocol;

import org.klomp.eclipse.team.sftp.SftpException;
import org.klomp.eclipse.team.sftp.client.SftpCommunicationException;
import org.klomp.eclipse.team.sftp.client.SftpServerException;
import org.klomp.sftp.packet.FxpClose;
import org.klomp.sftp.packet.FxpStatus;
import org.klomp.sftp.packet.SftpPacket;
import org.klomp.sftp.packet.SftpPacketFactory;

public class Operations {
    protected final SftpPacketFactory pktFactory;
    protected final SftpConnection connection;

    protected Operations(SftpConnection connection) {
        this.connection = connection;
        this.pktFactory = connection.getPktFactory();
    }

    /**
     * Handles a response by throwing an appropriate exception.
     * @throws SftpException if the response code is OK
     */
    protected static void defaultStatusHandler(FxpStatus status) throws SftpException {
        switch (status.getCode()) {
            case FxpStatus.FX_OK :
                break;
            default :
                //  System.err.println(
                //      "Error status, code=" + status.getCode() + ", message=" + status.getMessage());
                throw new SftpServerException(status.getMessage(), status.getCode());
        }
    }

    protected FxpStatus readStatus(int id) throws SftpException {
        SftpPacket packet = connection.readPacket(id);
        if (packet.getType() != SftpPacket.FXP_STATUS) {
            throw new SftpCommunicationException(
                "Status packet expected but received: " + packet,
                null);
        }
        return (FxpStatus) packet;
    }

    protected void closeHandle(byte[] handle) throws SftpException {
        FxpClose pkt = pktFactory.createClose();
        pkt.setHandle(handle);

        connection.sendPacket(pkt);
        // Very naughty: don't wait for the status response to a close message, but tell the connection
        // to ignore it. That speeds things up, and generally one doesn't know what to do with a failed
        // close anyway. TOXDO: allow the client code to tell if this method should wait for a response
        // or not.
        connection.setMaxIgnorePacketId(pkt.getId());
        //FxpStatus response = readStatus(pkt.getId());
        //defaultStatusHandler(response);
    }
}
